#!/usr/bin/env python
from dataclasses import dataclass, field
from pathlib import Path
from typing import List
import argparse
import logging
import mimetypes
import os

from . import utils
from . import thumbnail

from argparse_utils import enum_action, mapping_action
from PIL import Image
import jinja2

XDG_SIZE = utils.XDG_SIZE

logger = logging.getLogger(__name__)


@dataclass
class Output:
    message: str


@dataclass
class MIME:
    mime: str
    split: bool = True

    @property
    def valid(self):
        return self.mime is not None and '/' in self.mime

    def __str__(self):
        return self.mime

    @property
    def type(self):
        if self.valid and self.split:
            return self.mime.split('/')[0]
        else:
            # Should return "None" when invalid
            return self.mime


@dataclass(frozen=True)
class GlobalData:
    """
    Data global to whole task list. Should be passed to all templates.
    """
    title: str
    insecure: bool
    static: str  # URL


@dataclass(frozen=True)
class TaskInput:
    fname: Path
    mime: MIME


@dataclass
class Task:
    message: str
    mime_types: List[str]
    inputs: List[TaskInput] = field(init=False, default_factory=list)
    outputs: List[object] = field(init=False, default=None)

    @property
    def todo(self):
        return len(self.inputs) > 0

    def doit(self, **kwargs):
        """
        Internal entrypoint for the task.

        DO NOT OVERRIDE ON subclasses, see `do_all`
        """
        self.do_pre(**kwargs)
        self.outputs = self.do_all(**kwargs)
        self.do_post(**kwargs)

    def do_pre(self, **kwargs):
        """
        Setup Task. Runs before `do_all`.

        Can be overriden on sub-classes.
        """
        pass

    def do_post(self, **kwargs):
        """
        Tear Down Task. Runs after `do_all`.

        Can be overriden on sub-classes.
        """
        pass

    def do_all(self, **kwargs):
        """
        Main entrypoint for the task. Defaults to running `do_single` for each
        input, stateless.

        Can be overriden on sub-classes.
        """
        return [self.do_single(inp, **kwargs) for inp in self.inputs]

    def do_single(self, inp: TaskInput, **kwargs):
        """
        Entrypoint for simple stateless tasks. Called by `do_all` for each
        input.
        """
        raise NotImplementedError


class TaskThumbnails(Task):
    def __init__(self):
        super().__init__('Thumbnails', mime_types=[
            'image',
            'video',
        ])
        self.creators = {
            'image': thumbnail.image,
            'video': thumbnail.video,
        }

    def do_pre(self, *, args, **kwargs):
        dir_mod, self.file_mod = (0o705, 0o604) if args.public else (0o700, 0o600)

        toplevel = args.basedir / utils.XDG_SHARED_FOLDER
        logger.debug('Setup directories @ %s', toplevel)
        utils.mkdir(toplevel, dir_mod)
        for subdir in ['normal', 'large', 'fail']:
            subpath = toplevel / subdir
            utils.mkdir(subpath, dir_mod)

    def do_single(self, inp, *, sizes, largest_size, **kwargs):
        logger.debug('- %s', inp.fname)
        creator = self.creators[inp.mime.type]
        result = dict()
        for size in sizes:
            result[size] = self.do_thumbnail(inp.fname, size, creator, **kwargs)
        if None in result.values():
            logger.error('Broken File: %s', inp.fname)
        return result[largest_size]  # Show only the biggest size on the HTML

    def do_thumbnail(self, ifname, size, creator, *, args, **kwargs):
        tpath = utils.thumbnail_path(ifname, size, args.format, basedir=args.basedir)
        logger.debug('Thumbnail: %s', tpath)
        existing = tpath.exists()
        result = True
        if existing:
            logger.debug('- Thumbnail exists!')
            os.chmod(tpath, self.file_mod)
        if args.force or not existing:
            logger.debug('- Saving ...')
            try:
                result = creator.thumbnail(ifname, tpath,
                                           size=size,
                                           fmt=args.format,
                                           template=args.template,
                                           verbose=args.verbose,
                                           # MIME Type-specific
                                           overlay=args.thumb_overlay)
                logger.debug('- Saved!')
            except Exception as e:
                logger.debug('- Failed!')
                logger.debug('=> %r', e)
                # TODO: Write to the /fail/ directory
                # See https://specifications.freedesktop.org/thumbnail-spec/thumbnail-spec-latest.html#FAILURES
                result = None  # Explicit
        else:
            logger.debug('- Skipping!')
        if result is not None:
            result = creator.info(ifname, tpath,
                                  size=size,
                                  fmt=args.format,
                                  verbose=args.verbose)

        return result

    def do_post(self, *, jinja, args, gdata, largest_size, **kwargs):
        template = jinja.get_template('photo-album.html')
        logger.debug('HTML Template @ "%s"', template.filename)

        htmlname = args.basedir / 'index.html'
        logger.debug('HTML @ "%s"', htmlname.resolve())

        # Ignore files with errors
        with htmlname.open(mode='w') as htmlfile:
            s = template.stream({
                'gdata': gdata,
                'thumbnails': [obj for obj in self.outputs if obj is not None],
                'maxsize': '%d' % largest_size.value,
            })
            s.dump(htmlfile)


class TaskTracks(Task):
    def __init__(self):
        super().__init__('Thumbnails', mime_types=[
            'application/gpx+xml'
        ])

    def do_single(self, inp: TaskInput, args, **kwargs):
        # TODO: Parse the XML for more metadata
        # https://www.topografix.com/GPX/1/1/#type_metadataType
        # XPath: /metadata/name
        return {
            'title': inp.fname.stem,
            'url': inp.fname.relative_to(args.basedir),
        }

    def do_post(self, *, jinja, args, gdata, **kwargs):
        template = jinja.get_template('tracks.html')
        logger.debug('HTML Template @ "%s"', template.filename)

        htmlname = args.basedir / 'tracks.html'
        logger.debug('HTML @ "%s"', htmlname.resolve())

        with htmlname.open(mode='w') as htmlfile:
            s = template.stream({
                'gdata': gdata,
                'default_tile': args.gpx_defaulttile,
                'tracks': self.outputs,
            })
            s.dump(htmlfile)


OUTPUT_PATHS = {
    '.sh_thumbnails': Output('Shared Thumbnails'),
    'index.html': Output('Main Index'),
    'tracks.html': Output('GPX Tracks'),
}

MIMES = {
    'inode/directory': MIME('inode/directory', split=False),
}
EXT_MIMES = {
    '.gpx': MIME('application/gpx+xml', split=False),
}

TASKS = [
    TaskTracks(),
    # TODO: Task('Recursive Invocation', mime_types=['inode/directory']),
    TaskThumbnails(),
]

# e: Path
SORT_KEY = {
    'name': lambda e: e.name,
    'mtime': lambda e: e.stat().st_mtime,
}


def Directory(location):
    location = Path(location)
    if location.is_dir():
        return location
    else:
        raise argparse.ArgumentTypeError(f'Invalid folder: {location}')


def __main__():
    parser = argparse.ArgumentParser(
        description='Publish static HTML to describe a folder',
        allow_abbrev=False
    )
    parser.add_argument('--basedir',
                        type=Directory, default=Path('.'),
                        help='Base directory to process. Defaults to `%(default)s`')
    parser.add_argument('--sort', action=mapping_action(SORT_KEY),
                        default='name',
                        help='Sort files by metadata. Defaults to `%(default)s`')
    parser.add_argument('--title', default=None,
                        help='Publishing Title. Default to the `--basedir` name')
    parser.add_argument('-f', '--force',
                        action='store_true',
                        help='Overwrite existing thumbnails')
    parser.add_argument('-p', '--public',
                        action='store_true',
                        help='Adjust the file permissions for public read-only access. '
                             'Defaults to the specification, exclusive owner access')
    parser_size = parser.add_mutually_exclusive_group()
    parser_size.add_argument('-s', '--size', action=enum_action(XDG_SIZE),
                             help='Create the given size.')
    parser_size.add_argument('-S', '--sizes-all',
                             action='store_true',
                             help='Create all available sizes')
    parser.add_argument('-m', '--format', type=lambda f: f.value,
                        action=enum_action(utils.Format),
                        default=utils.Format.thumbnails_default.value,
                        help='The image format for thumbnails. Defaults to %(default)s')
    parser.add_argument('--template', type=Path,
                        action='append', default=[],
                        help='Extra template location to use. Overrides other locations')
    parser.add_argument('--static-location', default='',
                        help='The URL (absolute or relative) where the files in `static` are provided. Defaults to being relative to the HTML location')
    parser.add_argument('--no-thumb-play-overlay', dest='thumb_overlay',
                        action='store_const', default=True, const=None,
                        help='Thumbnails: Hide the "play" overlay image. Defaults to an internal one, based on size')
    parser.add_argument('--gpx-defaulttile', default='OpenStreetMap',
                        help='Tracks: The default tile data key (see `static/gpx/multiple.js`). Defaults to %(default)s')
    parser.add_argument('--insecure', action='store_true',
                        help='Do not use any security-related features on the generated files. Useful for tests on `localhost`')
    parser.add_argument('-v', '--verbose',
                        help='Show debug messages',
                        action='store_true')
    args = parser.parse_args()

    utils.setup_logging(args.verbose, shutup=[
        'PIL',
    ])

    Image.init()  # Initialize PIL
    mimetypes.init()  # Initialize MIME types

    # Mapper MIME -> Task
    task_mapper = dict()
    for t in TASKS:
        for mtype in t.mime_types:
            # On conflict, first task wins
            # TODO: Support same file to multiple tasks? Not needed so far
            task_mapper.setdefault(mtype, t)

    # File Selection
    logger.debug('Base Directory: "%s"', args.basedir)
    for element in sorted(args.basedir.iterdir(), key=args.sort):
        if output := OUTPUT_PATHS.get(element.name):
            logger.debug('= "%s": %s', element, output.message)
            continue
        if element.is_dir():
            # Use this as MIME for a directory
            # See: https://specifications.freedesktop.org/shared-mime-info-spec/shared-mime-info-spec-latest.html#idm140625828597376
            mimetype = MIMES['inode/directory']
        else:
            # Discover MIME type based on file name
            raw_mimetype = mimetypes.guess_type(element)
            mimetype = raw_mimetype[0]
            if mimetype is None:
                # TODO: Sniff the file contents? Use `file --mime-type`?
                # No automatic detection, fallback to EXT_MIMES
                mimetype = EXT_MIMES.get(''.join(element.suffixes))
            else:
                # Check the MIMES for custom MIME objects
                mimetype = MIMES.get(mimetype, MIME(mimetype))
        logger.debug('- "%s": %s', element, mimetype)
        # `mimetype` can be None
        if mimetype and mimetype.type in task_mapper:
            telement = TaskInput(element, mimetype)
            # TODO: Turn into generator
            task_mapper[mimetype.type].inputs.append(telement)

    # Size Selection
    if args.sizes_all:
        sizes = [s for s in XDG_SIZE]
    else:
        sizes = args.size or [XDG_SIZE.large]  # Default to building only the "large" thumbnails
    largest_size = sorted(sizes, key=lambda s: s.value, reverse=True)[0]
    logger.debug('Largest Size: %s', largest_size)

    # Global Values
    gdata = GlobalData(
        title=args.title or args.basedir.resolve().name,
        insecure=args.insecure,
        static=args.static_location,
    )

    # Jinja2
    jinja = jinja2.Environment(
        loader=utils.JinjaTemplateLoader(*args.template),
        autoescape=jinja2.select_autoescape(('html', 'xml')),
    )

    logger.debug('Tasks')
    for task in TASKS:
        if task.todo:
            logger.debug('- %s| %d items', task.message, len(task.inputs))
            # for tin in task.inputs:
            #     logger.debug('  > %s| %s', tin.mime.type, tin.fname)
            task.doit(**locals())  # This is a very ugly hack...
            # for tout in task.outputs:
            #     logger.debug('  < %r', tout)


if __name__ == '__main__':
    import sys
    sys.exit(__main__())



#!/usr/bin/env python
from pathlib import Path
from shutil import rmtree
import argparse
import logging

from . import pypublish

logger = logging.getLogger(__name__)


def __main__():
    parser = argparse.ArgumentParser(description='Remove "pypublish" creations')

    parser.add_argument('-f', '--rm',
                        action='store_true',
                        help='Effectively delete the files')
    args = parser.parse_args()

    if not args.rm:
        logger.info('Dry-Run, listing only the files to delete')

    for o in pypublish.OUTPUT_PATHS:
        opath = Path(o)
        if opath.exists():
            print(opath)
            if args.rm:
                # Remove the file, or folder
                if opath.is_dir():
                    rmtree(opath)
                else:
                    opath.unlink()


if __name__ == '__main__':
    import sys
    sys.exit(__main__())



#!/usr/bin/env python
import logging
from enum import Enum, IntEnum
from pathlib import Path
from itertools import chain
from dataclasses import dataclass, field
from time import strftime
from hashlib import md5
from os.path import getmtime
import importlib.resources as res

from jinja2 import BaseLoader, TemplateNotFound


PROJECT_NAME = 'pythumbnailer'
logger = logging.getLogger(__name__)


def setup_logging(verbose, shutup=[]):
    _logging = {
        'fmt': '%(levelname).4s %(name)s@%(funcName)s: %(message)s',
    }
    if verbose:
        _logging['level'] = logging.DEBUG
    try:
        import coloredlogs
        coloredlogs.install(**_logging)
    except ModuleNotFoundError:
        _logging['format'] = _logging.pop('fmt')
        logging.basicConfig(**_logging)
    if verbose:
        # Silence spammy logs
        for mod in shutup:
            logging.getLogger(mod).setLevel(logging.WARNING)


def mkdir(path, dir_mod):
    if path.is_dir():
        # If it exists, make sure the permissions are correct
        path.chmod(dir_mod)
    else:
        # If it doesn't exist, create it
        path.mkdir(mode=dir_mod)


@dataclass(frozen=True)
class Resolution:
    width: int
    height: int

    @property
    def tpl(self):
        """
        Return value as tuple
        """
        return (self.width, self.height)


@dataclass(frozen=True)
class TimePoint:
    tpl: tuple  # Time Tuple
    accurate: bool  # Is this worth showing, or it just for internal consumption?

    def strftime(self, fmt='%Y-%m-%d %H:%M:%S'):
        return strftime(fmt, self.tpl)


# FreeDesktop Thumbnail Managing Standard
# https://specifications.freedesktop.org/thumbnail-spec/thumbnail-spec-latest.html#DIRECTORY
class XDG_SIZE(IntEnum):
    normal = 128
    large = 256

    @property
    # Always squares
    def res(self):
        """
        Return a Resolution object
        """
        return Resolution(width=self.value, height=self.value)


@dataclass(frozen=True)
class ImageFormat:
    """
    A supported image format.
    """
    extension: str
    mime: str
    # Settings for `PIL`
    pil_settings: dict = field(default_factory=dict)
    # `-f` for FFMPEG. Defaults to the `extension`
    ffmpeg_codec: str = None
    # PIL format. Defaults to `extension` in upper-case
    # https://pillow.readthedocs.io/en/stable/handbook/image-file-formats.html
    pil_format: str = None

    def __post_init__(self):
        # Setup the default FFMPEG codec
        object.__setattr__(self, 'ffmpeg_codec', self.ffmpeg_codec or self.extension)
        # Setup the default PIL format
        object.__setattr__(self, 'pil_format', self.pil_format or self.extension.upper())


class Format(Enum):
    JPEG = ImageFormat('jpg',
                       ffmpeg_codec='mjpeg',
                       pil_settings={
                           'optimize': True,
                           'progressive': True,  # Progressive thumbnails
                           'quality': 75,  # /100
                       },
                       mime='image/jpeg')
    PNG = ImageFormat('png',
                      pil_settings={
                          'optimize': True,
                      },
                      mime='image/png')

    def __str__(self):
        return self.name


# FreeDesktop Thumbnail Managing Standard
# Standard File Format is PNG
# https://specifications.freedesktop.org/thumbnail-spec/thumbnail-spec-latest.html#CREATION
Format.thumbnails_default: Format = Format.PNG


# TODO: Support XDG Base Directory Standard
TEMPLATES_LGLOBAL = Path('/usr/share')
TEMPLATES_LUSER = Path('~/.local/share').expanduser()
TEMPLATES_LOCATIONS = [
    TEMPLATES_LGLOBAL.joinpath(PROJECT_NAME, 'templates'),
    TEMPLATES_LUSER.joinpath(PROJECT_NAME, 'templates'),
    res.path('pythumbnailer', 'templates').__enter__(),  # Current module Folder
]


def template_locations(*override: Path):
    """
    Return the default search paths, including the overriden locations.

    override: Add the given paths to the beginning of the search paths
    """
    return chain(override, TEMPLATES_LOCATIONS)


def template(fname: str, *override: Path):
    """
    Search for the template named `fname` on the search paths.

    This is for "template" file, non-jinja only.
    """
    # For Jinja templates, use `JinjaTemplateLoader`
    for location in template_locations(*override):
        possible = location.joinpath(fname)
        if possible.exists():
            logger.debug(f'Template[{fname}]: "{possible}"')
            return possible
    return None  # Not found


class JinjaTemplateLoader(BaseLoader):
    # https://jinja.palletsprojects.com/en/2.11.x/api/?highlight=choiceloader#jinja2.BaseLoader
    def __init__(self, *override: Path):
        self._overrides = override

    def get_source(self, environment, name):
        path = template(name, *self._overrides)
        if not path.exists():
            raise TemplateNotFound(name)
        mtime = getmtime(path)
        with path.open('r', encoding='utf-8') as obj:
            source = obj.read()
        return source, str(path), lambda: mtime == getmtime(path)


# FreeDesktop Thumbnail Managing Standard
# Shared Thumbnails Only
# https://specifications.freedesktop.org/thumbnail-spec/thumbnail-spec-latest.html#SHARED
XDG_SHARED_FOLDER = '.sh_thumbnails'


@dataclass(frozen=True)
class ThumbnailData:
    path: Path
    original: Path
    title: str  # Image "title", description, whatever you want to call it
    time: TimePoint
    size: Resolution = None


def _thumbnail_startpath(fname: Path, basedir: Path = None):
    return basedir or fname.parent


def thumbnail_hash(fname: Path, basedir: Path = None):
    """
    Get the XDG hash for the given filename. Implements
    https://specifications.freedesktop.org/thumbnail-spec/thumbnail-spec-latest.html#THUMBSAVE,
    for the shared thumbnails variant.

    fname: The Path to calculate the hash.
    basedir: The root Path to consider when calculating the name.
            When `None`, use the directory of the given `fname`.
            Defaults to `None`
    """
    start_path = _thumbnail_startpath(fname, basedir)
    to_hash = fname.relative_to(start_path)
    return md5(to_hash.as_posix().encode('utf-8')).hexdigest()


def thumbnail_path(fname: Path, size: XDG_SIZE, fmt: Format, basedir: Path = None):
    """
    Get the thumbnail path for the given filename and size, in the given format.

    fname: The Path to calculate the filename
    basedir: The root Path to consider when calculating the name.
            When `None`, use the directory of the given `fname`.
            Defaults to `None`
    size: The XDG_SIZE of the thumbnail
    fmt: The Format of the thumbnail
    """
    start_path = _thumbnail_startpath(fname, basedir)
    hexhash = thumbnail_hash(fname, basedir)
    tname = '%s.%s' % (hexhash, fmt.extension)
    return start_path / XDG_SHARED_FOLDER / size.name / tname



#!/usr/bin/env python
from pathlib import Path
import logging
import time

from .. import utils

from PIL import Image

XDG_SIZE = utils.XDG_SIZE
Format = utils.Format

logger = logging.getLogger(__name__)


EXIF_ORIENTATION = 0x0112  # PIL.ExifTags.TAGS with Orientation
EXIF_DATETIME = 0x0132  # PIL.ExifTags.TAGS with DateTime
ORIENTATION_OP = {
    1: [],  # Default
    2: [Image.FLIP_LEFT_RIGHT],
    3: [Image.ROTATE_180],
    4: [Image.FLIP_TOP_BOTTOM],
    5: [Image.FLIP_LEFT_RIGHT, Image.ROTATE_90],
    6: [Image.ROTATE_270],
    7: [Image.FLIP_TOP_BOTTOM, Image.ROTATE_270],
    8: [Image.ROTATE_90],
}


def image_size(image: Image):
    return utils.Resolution(width=image.width, height=image.height)


def image_time(image: Image, ipath: Path):
    # Information taken from:
    # - EXIF tags
    # - File Creation Time
    o_time_exif = None
    # Only if EXIF information is exposed
    if getattr(image, '_getexif'):
        raw_exif = image._getexif()
        if raw_exif is not None:
            dt = raw_exif.get(EXIF_DATETIME, None)
            if dt:
                # Get date from EXIF
                # https://www.awaresystems.be/imaging/tiff/tifftags/privateifd/exif/datetimeoriginal.html
                try:
                    o_time_exif = time.strptime(dt, '%Y:%m:%d %H:%M:%S')
                except ValueError:
                    # Invalid EXIF format, skip
                    o_time_exif = None
    return utils.TimePoint(
        tpl=o_time_exif or time.gmtime(ipath.stat().st_ctime),
        accurate=o_time_exif is not None
    )


def image_title(image: Image, ipath: Path):
    # File Name, without extension
    # TODO: Get something title/description from EXIF?
    # - https://www.awaresystems.be/imaging/tiff/tifftags/privateifd/exif/usercomment.html
    #   - ImageDescription (ASCII only)
    #   - UserComment: Supports UTF-8 (write only this)
    return ipath.stem


def thumbnail(ifile: Path, ofile: Path, *, size: XDG_SIZE, fmt: Format, template=[], verbose=False, **options):
    image = Image.open(ifile)
    o_size = image_size(image)
    o_time = image_time(image, ifile)
    o_title = image_title(image, ifile)
    ops = []
    # Only if EXIF information is exposed
    if getattr(image, '_getexif'):
        raw_exif = image._getexif()
        if raw_exif is not None:
            # Orientation
            orientation = raw_exif.get(EXIF_ORIENTATION, None)
            if orientation in ORIENTATION_OP:
                ops.extend(ORIENTATION_OP[orientation])
            logger.debug('Orientation: %r', orientation)
    # Modify the image in-place (in RAM, doesn't change the ifile)
    # TODO: Same thing as the video, pad the image to the exact size with transparent pixels
    image.thumbnail(size.res.tpl)
    logger.debug('Transpositions: %r', ops)
    for op in ops:
        image = image.transpose(op)
    with open(ofile, mode='wb') as ofobj:
        image.save(ofobj, fmt.pil_format, **fmt.pil_settings)
    return utils.ThumbnailData(
        path=ofile,
        original=ifile,
        title=o_title,
        size=o_size,
        time=o_time,
    )


def info(ifile: Path, ofile: Path, *, size: XDG_SIZE, fmt: Format, template=[], verbose=False, **options):
    image = Image.open(ifile)
    return utils.ThumbnailData(
        path=ofile,
        original=ifile,
        title=image_title(image, ifile),
        size=image_size(image),
        time=image_time(image, ifile),
    )


if __name__ == '__main__':
    import argparse

    parser = argparse.ArgumentParser(description='Create thumbnail for video files')

    parser.add_argument('file', type=Path,
                        help='The input file name to thumbnailize')
    parser.add_argument('output', type=Path,
                        help='The thumbnail file name')
    parser.add_argument('-s', '--size', type=utils.Enum_type(XDG_SIZE),
                        choices=utils.Enum_choices(XDG_SIZE),
                        default=XDG_SIZE.large,
                        help='Create the given size. Defaults to %(default)s')
    parser.add_argument('-m', '--format', type=utils.Enum_type(Format, only_value=True),
                        choices=utils.Enum_choices(Format),
                        default=Format.thumbnails_default,
                        help='The image format for thumbnails. Defaults to %(default)s')
    parser.add_argument('--template', type=Path,  # Un-used, worth to keep around
                        action='append', default=[],
                        help='Extra template location to use. Overrides other locations')
    parser.add_argument('-v', '--verbose',
                        action='store_true',
                        help='Show debug messages')
    args = parser.parse_args()

    utils.setup_logging(args.verbose)

    thumbnail(args.file, args.output,
              size=args.size,
              fmt=args.format,
              template=args.template,
              verbose=args.verbose,
              )



#!/usr/bin/env python
from pathlib import Path
import os
import argparse
import logging
import subprocess

from . import pypublish
from . import utils

logger = logging.getLogger(__name__)


def __main__():
    parser = argparse.ArgumentParser(description='Remove "pypublish" creations')

    parser.add_argument('--basedir',
                        type=pypublish.Directory, default=Path('.'),
                        help='Base directory to process. Defaults to `%(default)s`')
    parser.add_argument('-v', '--verbose',
                        action='store_true',
                        help='Show debug messages')
    args = parser.parse_args()

    utils.setup_logging(args.verbose)

    for p in args.basedir.iterdir():
        if p.is_file():
            if p.suffix == '.JPG':
                new_name = p.with_suffix('.jpg')
                if not new_name.exists():
                    logger.debug(f'JPG: Rename "{p}" -> "{new_name}"')
                    p.rename(new_name)
                else:
                    new_name = None
            elif p.suffix.lower() == '.mov':
                new_name = p.with_suffix('.mp4')
                if not new_name.exists():
                    loglevel = ['-v', 'quiet']
                    if args.verbose:
                        loglevel = []
                    convert_command = [
                        'ffmpeg',
                        *loglevel,
                        '-i', p,
                        '-movflags', 'use_metadata_tags',
                        new_name,
                        '-map_metadata', '0',
                    ]
                    logger.debug(f'MOV: Convert "{p}" -> "{new_name}"')
                    convert_ret = subprocess.call(convert_command)
                    if convert_ret == 0:
                        logger.debug('- Success!')
                        old_stat = p.stat()
                        old_times = (old_stat.st_atime, old_stat.st_mtime)
                        os.utime(new_name, times=old_times)  # touch --reference=
                        p.unlink()
                    else:
                        logger.debug('- Error!')
                else:
                    new_name = None  # Skip


if __name__ == '__main__':
    import sys
    sys.exit(__main__())



import setuptools

with open('README.md', 'r') as rdm:
    long_description = rdm.read()

setuptools.setup(
    name='pythumbnailer',
    version='0.4',
    author='somini',
    author_email='dev@somini.xyz',
    description='Create static HTML galleries with thumbnails',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://gitlab.com/somini/pythumbnailer/',
    classifiers=[
        'Environment :: Console',
        'Environment :: Web Environment',
        'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
    ],
    packages=setuptools.find_packages(),
    python_requires='>=3.8',
    install_requires=[  # Dependencies
        'jinja2',
        'pillow',
        'argparse_utils',
    ],
    entry_points={
        'console_scripts': [
            'pypublish = pythumbnailer.pypublish:__main__',
            'pypublish-clean = pythumbnailer.clean:__main__',
            'pypublish-prepare = pythumbnailer.prepare:__main__',
        ],
    },
    package_data={
        'pythumbnailer': [
            'static/*',
            'templates/*',
        ],
    },
)



#!/usr/bin/env python
from datetime import timedelta
from itertools import chain
from pathlib import Path
from time import gmtime
import json
import logging
import subprocess

from .. import utils

XDG_SIZE = utils.XDG_SIZE
Format = utils.Format

logger = logging.getLogger(__name__)


def _probe(ifile: Path, verbose: bool):
    loglevel = ['-v', 'quiet']
    if verbose:
        loglevel = []
    probe_command = [
        'ffprobe',
        *loglevel,
        '-print_format', 'json',
        '-show_format',
        '-show_streams',
        ifile,
    ]
    json_text = subprocess.check_output(probe_command, text=True)
    return json.loads(json_text)


def video_title(probe, ipath: Path):
    # TODO: Get this from some video tag ???
    return ipath.stem


def video_size(probe):
    # Usually there's only a single video stream, but there might be more...
    w = max([s['width'] for s in probe['streams'] if s['codec_type'] == 'video'], default=None)
    h = max([s['height'] for s in probe['streams'] if s['codec_type'] == 'video'], default=None)
    if None in (w, h):
        return None
    else:
        return utils.Resolution(width=w, height=h)


def video_time(probe, ipath: Path):
    # TODO: Get this from some video tag ???
    return utils.TimePoint(
        tpl=gmtime(ipath.stat().st_ctime),
        accurate=False,
    )


def thumbnail(ifile: Path, ofile: Path, *, size: XDG_SIZE, fmt: Format, template=[], verbose=False, overlay=None, **options):
    loglevel = ['-v', 'quiet']
    if verbose:
        loglevel = []
    probe = _probe(ifile, verbose)
    raw_duration = probe['format']['duration']
    duration_seconds = round(float(raw_duration))
    duration_string = str(timedelta(seconds=duration_seconds))
    logger.debug('Duration: "%s" => "%s"', raw_duration, duration_string)

    inputs = [ifile]
    filters = [
        'thumbnail',
        # Scale towards the resolution, keeping the aspect ratio
        'scale=%s' % ':'.join([
            "'min(%d, iw)':'min(%d, ih)'" % size.res.tpl,
            'force_original_aspect_ratio=decrease',
        ]),
        # Pad the rest of the size with transparent pixels
        'pad=%s' % ':'.join([
            '%d:%d' % size.res.tpl,
            'x=(ow-iw)/2',
            'y=(oh-ih)/2',
            "color=0x00000000",  # Transparent
        ]),
        'drawtext=%s' % ':'.join([
            # Center horizontally, on the bottom
            'x=(w-text_w)/2',
            'y=(h-line_h)-1',
            # Font Settings
            'fontsize=%d' % (size // 7),  # Should take about 1/4 of the height
            'borderw=1', 'bordercolor=black',
            'fontcolor=white',
            # Escape the text, twice
            'expansion=none',
            'text=%s' % duration_string.replace(':', r'\\:'),
        ]),
    ]
    if overlay is True:
        # Default to a size-specific play circle
        overlay = utils.template('play-circle-regular.%d.png' % int(size), *template)

    if overlay is not None:
        if not overlay.exists():
            raise Exception('Invalid overlay file: %s' % overlay)
        # Overlay the file, without processing
        inputs.append(overlay)
        filters.append('overlay=%s' % ':'.join([
            # Center overlay on the main frame
            'x=(main_w-overlay_w)/2',
            'y=(main_h-overlay_h)/2',
        ]))

    thumbnail_command = [
        'ffmpeg',
        '-y',  # Overwrite existing file
        *loglevel,
        *chain.from_iterable([['-i', f] for f in inputs]),
        '-filter_complex', ','.join(filters),
        '-frames:v', '1',  # Single frame
        # Output Format and Codec
        '-f', 'image2',
        '-codec', fmt.ffmpeg_codec,
        ofile,
    ]
    subprocess.run(thumbnail_command)
    return utils.ThumbnailData(
        path=ofile,
        original=ifile,
        title=video_title(probe, ifile),
        size=video_size(probe),
        time=video_time(probe, ifile),
    )


def info(ifile: Path, ofile: Path, *, size: XDG_SIZE, fmt: Format, template=[], verbose=False, overlay=None, **options):
    probe = _probe(ifile, verbose)
    return utils.ThumbnailData(
        path=ofile,
        original=ifile,
        title=video_title(probe, ifile),
        size=video_size(probe),
        time=video_time(probe, ifile),
    )


if __name__ == '__main__':
    import argparse
    import shutil

    parser = argparse.ArgumentParser(description='Create thumbnail for video files')

    parser.add_argument('file', type=Path,
                        help='The input file name to thumbnailize')
    parser.add_argument('output', type=Path,
                        help='The thumbnail file name')
    parser.add_argument('-s', '--size', type=utils.Enum_type(XDG_SIZE),
                        choices=utils.Enum_choices(XDG_SIZE),
                        default=XDG_SIZE.large,
                        help='Create the given size. Defaults to %(default)s')
    parser.add_argument('-m', '--format', type=utils.Enum_type(Format, only_value=True),
                        choices=utils.Enum_choices(Format),
                        default=Format.thumbnails_default,
                        help='The image format for thumbnails. Defaults to %(default)s')
    parser.add_argument('--template', type=Path,
                        action='append', default=[],
                        help='Extra template location to use. Overrides other locations')
    parser.add_argument('--play-overlay', type=Path,
                        default=True,
                        metavar='OVERLAY_FILE',
                        help='Override the "play" image to overlay on the thumbnail')
    parser.add_argument('-v', '--verbose',
                        action='store_true',
                        help='Show debug messages')
    args = parser.parse_args()

    utils.setup_logging(args.verbose)

    # Hard Requirements
    for cmd in ['ffmpeg', 'ffprobe']:
        if shutil.which(cmd) is None:
            parser.error(f'Missing executable: {cmd}')

    thumbnail(args.file, args.output,
              size=args.size,
              fmt=args.format,
              template=args.template,
              verbose=args.verbose,
              overlay=args.play_overlay,
              )
